///
///    This file is part of ILNumerics Community Edition.
///
///    ILNumerics Community Edition - high performance computing for applications.
///    Copyright (C) 2006 - 2012 Haymo Kutschbach, http://ilnumerics.net
///
///    ILNumerics Community Edition is free software: you can redistribute it and/or modify
///    it under the terms of the GNU General Public License version 3 as published by
///    the Free Software Foundation.
///
///    ILNumerics Community Edition is distributed in the hope that it will be useful,
///    but WITHOUT ANY WARRANTY; without even the implied warranty of
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
///    GNU General Public License for more details.
///
///    You should have received a copy of the GNU General Public License
///    along with ILNumerics Community Edition. See the file License.txt in the root
///    of your distribution package. If not, see <http://www.gnu.org/licenses/>.
///
///    In addition this software uses the following components and/or licenses: 
///
///    =================================================================================
///    The Open Toolkit Library License
///    
///    Copyright (c) 2006 - 2009 the Open Toolkit library.
///    
///    Permission is hereby granted, free of charge, to any person obtaining a copy
///    of this software and associated documentation files (the "Software"), to deal
///    in the Software without restriction, including without limitation the rights to 
///    use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
///    the Software, and to permit persons to whom the Software is furnished to do
///    so, subject to the following conditions:
///
///    The above copyright notice and this permission notice shall be included in all
///    copies or substantial portions of the Software.
///
///    =================================================================================
///    

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ILNumerics.Misc; 
using ILNumerics.Exceptions; 

namespace ILNumerics {
    public partial class ILMath {

        /// <summary>
        /// Create a random permutation of integers 
        /// </summary>
        /// <param name="length">Number of unique integers to be created</param>
        /// <returns>Vector of length 'length' and element type double</returns>
        public static ILRetArray<double> randperm(ILBaseArray length) {
            using (ILScope.Enter(length)) {
                if (object.Equals(length, null) || !length.IsScalar ||!length.IsNumeric) {
                    throw new ILArgumentException("the length parameter must be a positive integer");
                }
                int len = (int)toint32(length); 
                ILArray<double> ret = empty();
                sort(rand(1, len), ret, 1, false);
                return ret;
            }
        }
    }
}
