///
///    This file is part of ILNumerics Community Edition.
///
///    ILNumerics Community Edition - high performance computing for applications.
///    Copyright (C) 2006 - 2012 Haymo Kutschbach, http://ilnumerics.net
///
///    ILNumerics Community Edition is free software: you can redistribute it and/or modify
///    it under the terms of the GNU General Public License version 3 as published by
///    the Free Software Foundation.
///
///    ILNumerics Community Edition is distributed in the hope that it will be useful,
///    but WITHOUT ANY WARRANTY; without even the implied warranty of
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
///    GNU General Public License for more details.
///
///    You should have received a copy of the GNU General Public License
///    along with ILNumerics Community Edition. See the file License.txt in the root
///    of your distribution package. If not, see <http://www.gnu.org/licenses/>.
///
///    In addition this software uses the following components and/or licenses: 
///
///    =================================================================================
///    The Open Toolkit Library License
///    
///    Copyright (c) 2006 - 2009 the Open Toolkit library.
///    
///    Permission is hereby granted, free of charge, to any person obtaining a copy
///    of this software and associated documentation files (the "Software"), to deal
///    in the Software without restriction, including without limitation the rights to 
///    use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
///    the Software, and to permit persons to whom the Software is furnished to do
///    so, subject to the following conditions:
///
///    The above copyright notice and this permission notice shall be included in all
///    copies or substantial portions of the Software.
///
///    =================================================================================
///    

using System;
using System.Collections.Generic;
using System.Text;
using ILNumerics.Storage;
using ILNumerics.Misc;
using ILNumerics.Exceptions;


namespace ILNumerics  {
    public partial class ILMath {

        /// <summary>
        /// Real part of complex array
        /// </summary>
        /// <param name="X">Complex input array</param>
        /// <returns>Real part of complex array</returns>
        public static ILRetArray<double> real(ILInArray<complex> X) {
            using (ILScope.Enter(X)) {
                ILArray< double> ret = zeros<double>(X.Size);
                
                double[] retArr = ret.GetArrayForWrite();
                
                complex[] XArr = X.GetArrayForRead();
                int nrX = X.Size.NumberOfElements;
                for (int i = 0; i < nrX; i++) {
                    retArr[i] = XArr[i].real;
                }
                return ret;
            }
        }

#region HYCALPER AUTO GENERATED CODE

        /// <summary>
        /// Real part of complex array
        /// </summary>
        /// <param name="X">Complex input array</param>
        /// <returns>Real part of complex array</returns>
        public static ILRetArray<float> real(ILInArray<fcomplex> X) {
            using (ILScope.Enter(X)) {
                ILArray< float> ret = zeros<float>(X.Size);
               
                float[] retArr = ret.GetArrayForWrite();
               
                fcomplex[] XArr = X.GetArrayForRead();
                int nrX = X.Size.NumberOfElements;
                for (int i = 0; i < nrX; i++) {
                    retArr[i] = XArr[i].real;
                }
                return ret;
            }
        }

#endregion HYCALPER AUTO GENERATED CODE
   }

}
